/*
* File   : $Source: /usr/local/cvs/opencms/src/com/opencms/dbpool/Attic/CmsDriver.java,v $
* Date   : $Date: 2003/04/01 15:20:18 $
* Version: $Revision: 1.8 $
*
* This library is part of OpenCms -
* the Open Source Content Mananagement System
*
* Copyright (C) 2001  The OpenCms Group
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* For further information about OpenCms, please see the
* OpenCms Website: http://www.opencms.org
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package com.opencms.datasourcepool;

import java.sql.Connection;
import java.sql.DriverPropertyInfo;
import java.sql.SQLException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Properties;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

import com.opencms.boot.I_CmsLogChannels;
import com.opencms.core.A_OpenCms;
import com.opencms.core.I_CmsConstants;

import source.org.apache.java.util.Configurations;

/**
 * The Datasource database driver.
 * 
 * @author Dariush Behboudi
 * @see java.sql.Driver
 */
public class DataSourceDriver implements java.sql.Driver, I_CmsConstants, I_CmsLogChannels  {

    //
    // Register ourselves with the DriverManager
    //
    static {
		if(I_CmsLogChannels.C_LOGGING && A_OpenCms.isLogging() ) {
			A_OpenCms.log(I_CmsLogChannels.C_OPENCMS_POOL, ". Using DatasourceDriver..." );
		}
    	//System.out.println("istanzio il driver DataSourceDriver!!!!!");
        try {
			java.sql.DriverManager.registerDriver(new DataSourceDriver());
        } catch (java.sql.SQLException E) {
            throw new RuntimeException("Can't register driver!");
        }
    }

    /**
     * The start of the connection urls for this Driver
     */
    private static final String C_CONNECTION_URL = "jdbc:opencmspool:";
	private static int poolDebug = 0;
    /**
     * The configurations to read properties from.
     */
    private static Configurations c_config = null;

    /**
     * A hashtable to store all pools, identified by poolName.
     */
    private Hashtable ds_pools = new Hashtable();

    /**
     * Sets the confitgurations to read pool-properties from.
     * @param conf - the configurations to read from.
     */
    public static void setConfigurations(Configurations conf) {
        c_config = conf;
		poolDebug = c_config.getInteger("poolDebug");
		System.out.println("Enabling pool debug, level:" + poolDebug);
    }

    /**
     * Default-Constructor
     */
    private DataSourceDriver() {
        super();
    }

    /**
     * Try to make a database connection to the given URL. The driver should
     * return "null" if it realizes it is the wrong kind of driver to connect
     * to the given URL. This will be common, as when the JDBC driver manager
     * is asked to connect to a given URL it passes the URL to each loaded
     * driver in turn.
     *
     * The driver should raise a SQLException if it is the right driver to
     * connect to the given URL, but has trouble connecting to the database.
     *
     * The java.util.Properties argument can be used to passed arbitrary string
     *  tag/value pairs as connection arguments. Normally at least "user" and
     * "password" properties should be included in the Properties.
     * @param url - The URL of the database to connect to
     * @param info - a list of arbitrary string tag/value pairs as connection
     * arguments; normally at least a "user" and "password" property should be included
     * @return a Connection to the URL
     * @throws SQLException if a database-access error occurs.
     */
    public java.sql.Connection connect(String url, Properties info) 
        throws SQLException {
		if (poolDebug>2) {
			System.out.println ("DatasourceDriver, connect()");
		}
    //try {
        if(acceptsURL(url)) {
			if(I_CmsLogChannels.C_LOGGING && A_OpenCms.isLogging() ) {
				A_OpenCms.log(I_CmsLogChannels.C_OPENCMS_INIT, ". Database access init : ok" );
			}
			if (poolDebug>2) {
				System.out.println ("<<<<<<<<<<< url:" + url);
			}
            String poolName = url.substring(C_CONNECTION_URL.length());
			if (poolDebug>2) {
            	System.out.println ("poolName:" + poolName + " recupero il pool");
			}
            DataSource poolDs = getPool(poolName);
			if (poolDebug>2) {
				System.out.println("datasource retrived=" + poolDs +", asking for connection!!!!");
			}
			Connection connDs = poolDs.getConnection(); 
			if (poolDebug>2) {
				System.out.println("connection retrived=" + connDs );
			}
            return connDs;
        } else {
			if (poolDebug>2) {
	        	System.out.println ("url not recognized:" + url);
			}
            // this driver don't accepts this url - return null
            return null;
        }
	//} 
	/*catch (Exception e) {
		if (poolDebug) {
			System.out.println ("Errore in connect!!!!" + e);
		return null;
	}
	*/
    }

    /**
     * Returns true if the driver thinks that it can open a connection
     * to the given URL. Typically drivers will return true if they
     * understand the subprotocol specified in the URL and false if
     * they don't.
     * @param url - The URL of the database.
     * @return True if this driver can connect to the given URL.
     * @throws SQLException if a database-access error occurs.
     */
    public boolean acceptsURL(String url)
        throws SQLException {
        return url.startsWith(C_CONNECTION_URL);
    }

    /**
     * The getPropertyInfo method is intended to allow a generic GUI tool
     * to discover what properties it should prompt a human for in order to
     *  get enough information to connect to a database. Note that depending
     *  on the values the human has supplied so far, additional values may
     * become necessary, so it may be necessary to iterate though several
     *  calls to getPropertyInfo.
     *
     * @param url - The URL of the database to connect to.
     * @param info - A proposed list of tag/value pairs that will be sent
     * on connect open
     * @return An array of DriverPropertyInfo objects describing possible properties.
     *  This array may be an empty array if no properties are required.
     * @throws SQLException if a database-access error occurs.
     */
    public DriverPropertyInfo[] getPropertyInfo(String url, Properties info)
        throws SQLException {
        return new DriverPropertyInfo[0];
    }

    /**
     * Get the driver's major version number. Initially this should be 1.
     */
    public int getMajorVersion() {
        return 1;
    }

    /**
     * Get the driver's minor version number. Initially this should be 0.
     */
    public int getMinorVersion() {
        return 0;
    }

    /**
     * Report whether the Driver is a genuine JDBC COMPLIANT (tm) driver.
     * A driver may only report "true" here if it passes the JDBC compliance tests,
     * otherwise it is required to return false. JDBC compliance requires full
     * support for the JDBC API and full support for SQL 92 Entry Level. It is
     * expected that JDBC compliant drivers will be available for all the major
     * commercial databases. This method is not intended to encourage the
     * development of non-JDBC compliant drivers, but is a recognition of the fact
     * that some vendors are interested in using the JDBC API and framework for
     * lightweight databases that do not support full database functionality, or
     * for special databases such as document information retrieval where a SQL
     * implementation may not be feasible.
     */
    public boolean jdbcCompliant() {
        return false;
    }


	/**
	 * Returns the specified pool-object.
	 * @param poolName - the name of the pool to return
	 * @return Pool - the pool for this name
	 * @throws SQLException - if a SQL-Error occures.
	 */
	private DataSource getPool(String poolName) throws SQLException {
		if(!ds_pools.containsKey(poolName)) {
			if (poolDebug>0) {
				System.out.println ("Creating new pool:" + poolName);
			}
			ds_pools.put(poolName, createPool(poolName));
		}
		// return the existing or new created pool
		return (DataSource) ds_pools.get(poolName);
	}

	/**
	 * Creates a new pool with the name poolName. Reads informations
	 * for the pool from configurations.
	 * @param poolName - the name of the pool to create.
	 * @return Pool - the created pool.
	 * @throws SQLException - is a SQL-Error occurs.
	 */
	private DataSource createPool(String poolName) throws SQLException {
		String jndiPoolName = c_config.getString("dataSourcePrefix") + poolName; 
		if (poolDebug>0) {
			System.out.println("jndiPoolName:" + jndiPoolName);
		}
		InitialContext context = null;
		DataSource jdbcURL = null;
		Connection conn = null;
		try {
			if (poolDebug>0) {
				System.out.println ("creating InitialContext");
			}
			context = new InitialContext();
			if (poolDebug>0) {
				System.out.println ("Looking up" + jndiPoolName);
			}
			jdbcURL = (DataSource) context.lookup(jndiPoolName);
			return jdbcURL;		
		}
		catch(NamingException e)
		{
			if (poolDebug>0) {
				System.out.println ("NamingException:" + e);
			}
			return null;
		}
	}


    /**
     * Returns a string representation of this object.
     */
    public String toString() {
        StringBuffer output=new StringBuffer();
        output.append("[" + this.getClass().getName() + "]:");
        output.append(" available pools: \n");

        Enumeration keys = ds_pools.keys();

        while(keys.hasMoreElements()) {
            Object key = keys.nextElement();
            output.append("\t"+key + " :" + ds_pools.get(key).toString()+"\n");
        }
        return output.toString();
    }

    /**
     * Destroys this driver and all its connections.
     */
    public void destroy() {
		//Nothing, datasource think about that
    }
}
